# frozen_string_literal: true

require 'json'
require 'open3'

module Jekyll
  module Lunr
    # Error
    class Error < StandardError; end

    # Generates a Lunr index from documents
    class Indexer
      attr_reader :site, :lang, :fields

      def initialize(site:)
        @site = site
      end

      # The data is the register where Lunr looks for results.
      # TODO: Write a single data file per doc?
      def data
        @data ||= documents.map do |doc|
          doc.data.dup
            .keep_if { |k,_| fields.include? k }
            .merge(content: doc.content,
                   url: doc.url,
                   id: doc.url,
                   year: doc.date.year)
        end.compact
      end

      def data_file
        @data_file ||= Jekyll::StaticFile.new(
          site,
          site.source,
          '.',
          'data.json'
        )
      end

      # Convert data to strings since Lunr can't index objects
      # TODO: make elegant
      def indexable_data
        @indexable_data ||= data.map do |d|
          d.transform_values do |v|
            if v.is_a? Array
              if v.first.is_a? Hash
                v = v.map(&:values).join(', ')
              else
                v.join(', ')
              end
            elsif v.is_a? Hash
              v.values.join(', ')
            else
              v.to_s
            end
          end
        end
      end

      def write
        df = File.new(data_file.path, 'w')
        df.write data.to_json
        df.close

        site.static_files << data_file
      end

      def dir
        File.realpath(File.join([__dir__, '..', '..']))
      end

      def index_file
        @index_file ||= Jekyll::StaticFile.new(
          site,
          site.source,
          '.',
          'idx.json'
        )
      end

      def indexer
        "NODE_PATH=#{site.source}/node_modules node #{dir}/lib/assets/javascript/indexer.js #{lang}"
      end

      def index
        Open3.popen2(indexer) do |stdin, stdout, wait|
          stdin.puts indexable_data.to_json
          stdin.close

          idx = File.new(index_file.path, 'w')
          idx.write(stdout.read)
          idx.close

          site.static_files << index_file

          wait.value
        end
      end

      # Site lang
      def lang
        @lang ||= site.config.dig('lang')
      end

      # Indexable fields
      def fields
        @fields ||= ((site.config.dig('jekyll-lunr', 'fields') || []) + default_fields).uniq
      end

      def default_fields
        %w[title description]
      end

      # Indexable documents
      # TODO: make configurable
      def documents
        site.collections.values.map(&:docs).flatten
      end
    end
  end
end
